/*******************************************************************************
 * GenRGenS v.1.0, Generation of Random Genomic Sequences                      *
 * Copyright (C) 2001  Alain Denise/Yann Ponty                                 *
 *                                                                             *
 * This program is free software; you can redistribute it and/or               *
 * modify it under the terms of the GNU General Public License                 *
 * as published by the Free Software Foundation; either version 2              *
 * of the License, or (at your option) any later version.                      *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with this program; if not, write to the Free Software                 *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *******************************************************************************/

package GenRGenS.utility;

/******************************************************************************
*                        Classe FileExtensionFilter
*         
******************************************************************************/

import java.io.File;
import java.io.FilenameFilter;

/** 
 * Classe destine  la construction facilite de filtres
 * bass sur l'extension du fichier.
 * @author Fr&eacute;d&eacute;ric Sarron
 * @version 1.0
 * @see FilenameFilter
 */

public class FileExtensionFilter 
		implements FilenameFilter
{
		protected String extension;

		/**
		 * Construit une nouvelle instance de <code>FileExtensionFilter</code>
		 * dont le rle est d'accepter ou non des fichiers selon leur suffixe.
		 * @param Le suffixe requis pour que le ficheir soit accept.
		 */
		public FileExtensionFilter(String ext)
		{ extension = ext; };
		
		/**
		 * Dfinit si un fichier dcrit par son nom est son chemin d'accs
		 * est accept ou pas par le filtre.
     * @param dir Le repertoire du fichier
     * @param name Le nom du fichier
     * @see FilenameFilter
		 */
		public boolean accept(File dir,String name)
		{
				// dir is required by interface FileNameFilter
				// but is here irrelevant
				return(name.endsWith("."+extension));
		};
		
};
